local T, change_target = ...
local DEBUG_TOUCH = false

-- TODO If you want to check whether the device has a touch screen you should check whether HasTouchScreen is true.

local timer = Timer()

function T:CreateRegions()
	if (DEBUG_TOUCH) then
	elseif (not gkinterface.HasTouchScreen()) then return
	end

	local row = self.UI.display[1 + T.list_offset]
	local lr_x = tonumber(row.x)
	local lr_y = tonumber(row.y)
	local lr_w = tonumber(row.w)
	local lr_h = tonumber(row.h)
	local lr_h = lr_h * T.list_count
	--                                                                     1    2    3    4    5    6    7    8    9    10    11    12           13           14
	self.Regions.list = self.Regions.list or gkinterface.CreateTouchRegion(nil, nil, nil, nil, nil, nil, nil, nil, nil, lr_x, lr_y, lr_x + lr_w, lr_y + lr_h, -1)
	if (DEBUG_TOUCH) then
		local canvas = iup.canvas{size = (lr_w).."x"..(lr_h), cx = lr_x, cy = lr_y, border = "YES", bgcolor = "255 0 0", expand = "NO"}
		iup.Append(iup.GetNextChild(HUD.cboxlayer), canvas)
	end

	local fcolumn = self.UI.filters[1][2]
	local fr_x = tonumber(fcolumn.x)
	local fr_y = tonumber(fcolumn.y)
	local fr_w = tonumber(fcolumn.w)
	fr_w = fr_w * 4
	local fr_h = tonumber(fcolumn.h)
	--                                                                         1    2    3    4    5    6    7    8    9    10    11    12           13           14
	self.Regions.filter = self.Regions.filter or gkinterface.CreateTouchRegion(nil, nil, nil, nil, nil, nil, nil, nil, nil, fr_x, fr_y, fr_x + fr_w, fr_y + fr_h, -1)
	if (DEBUG_TOUCH) then
		local canvas = iup.canvas{size = (fr_w).."x"..(fr_h), cx = fr_x, cy = fr_y, border = "YES", bgcolor = "0 255 0", expand = "NO"}
		iup.Append(iup.GetNextChild(HUD.cboxlayer), canvas)
	end

	local tleft = self.UI.tabs[1][2]
	local tl_x = tonumber(tleft.x)
	local tl_w = tonumber(tleft.w)
	local tcolumn = self.UI.tabs[2][2]
	local tr_x = tonumber(tcolumn.x)
	local tr_y = tonumber(tcolumn.y)
	local tr_w = tonumber(tcolumn.w)
	tr_w = tr_w * 4
	local tr_h = tonumber(tcolumn.h)
	--                                                                     1    2    3    4    5    6    7    8    9    10    11    12           13           14
	self.Regions.tab      = self.tab      or gkinterface.CreateTouchRegion(nil, nil, nil, nil, nil, nil, nil, nil, nil, tr_x, tr_y, tr_x + tr_w, tr_y + tr_h, -1)
	if (DEBUG_TOUCH) then
		local canvas    = iup.canvas{size = (tr_w).."x"..(tr_h), cx = tr_x, cy = tr_y, border = "YES", bgcolor = "0 0 255", expand = "NO"}
		iup.Append(iup.GetNextChild(HUD.cboxlayer), canvas)
	end

	if (DEBUG_TOUCH) then iup.Map(HUD.cboxlayer) end
end

function T:DestroyRegions()
	if (not gkinterface.HasTouchScreen()) then return end

	gkinterface.DestroyTouchRegion(self.Regions.list)
	self.Regions.list = nil
	gkinterface.DestroyTouchRegion(self.Regions.filter)
	self.Regions.filter = nil
	gkinterface.DestroyTouchRegion(self.Regions.tab)
	self.Regions.tab = nil
end

-- Abort loading the rest of the file if not in TouchMode
if (not gkinterface.HasTouchScreen()) then return end

local longTimer = Timer()
local initial_p = 0

local function list_release(event, id, percentW, percentH)
	if (longTimer:IsActive()) then
		longTimer:Kill()
	elseif (not longTimer:IsActive() and event ~= "LONG_PRESS") then
		return
	end

	local row_count = T.list_count
	local row_h = T.UI.display[1].h
	
	if (initial_p > percentH) then
		-- Swipe Up
		if ((initial_p - percentH) < (1 / row_count)) then
			-- Tap
			percentH = initial_p
		else
			gkinterface.GKProcessCommand("targetprev")
		end
	else
		-- Swipe Down
		if ((percentH - initial_p) < (1 / row_count)) then
			-- Tap
			percentH = initial_p
		else
			gkinterface.GKProcessCommand("targetnext")
		end
	end
		
	if (percentH == initial_p) then
		for i = 1, row_count do
			if (percentH > ((i - 1) / row_count) and percentH < (i / row_count)) then
				if (event == "LONG_PRESS") then
					gkinterface.GKProcessCommand("pintarget "..i)
				else
					change_target(i)
				end
				break
			end
		end
	end
	initial_p = 0
end

local function list_press(event, id, percentW, percentH)
	longTimer:SetTimeout(500, function() list_release("LONG_PRESS", id, percentW, percentH) end)
end

local function filter_release(event, id, percentW, percentH)
	local column_count = 4
	local column_w = T.UI.filters[1][2].w

	if (initial_p > percentW) then
		-- Swipe Left
		if ((initial_p - percentW) < (1 / column_count)) then
			-- Tap
			percentW = initial_p
		else
			gkinterface.GKProcessCommand("cyclefilter "..(T.filter - 1))
		end
	else
		-- Swipe Right
		if ((percentW - initial_p) < (1 / column_count)) then
			-- Tap
			percentW = initial_p
		else
			gkinterface.GKProcessCommand("cyclefilter")
		end
	end
		
	if (percentW == initial_p) then
		for i = 1, column_count do
			if (percentW > ((i - 1) / column_count) and percentW < (i / column_count)) then
				if (T.filter == i) then
					gkinterface.GKProcessCommand("cyclefilterpage")
				else
					gkinterface.GKProcessCommand("cyclefilter "..(i))
				end
				break
			end
		end
	end
	initial_p = 0
end

local function tab_release(event, id, percentW, percentH)
	local timer = Timer()
	local column_count = T.tab_window_size
	local column_w = T.UI.tabs[2][2].w

	if (initial_p > percentW) then
		-- Swipe Left
		if ((initial_p - percentW) < (1 / column_count)) then
			-- Tap
			percentW = initial_p
		else
			timer:SetTimeout(1, function() gkinterface.GKProcessCommand("tabprev") end)
		end
	else
		-- Swipe Right
		if ((percentW - initial_p) < (1 / column_count)) then
			-- Tap
			percentW = initial_p
		else
			timer:SetTimeout(1, function() gkinterface.GKProcessCommand("tabnext") end)
		end
	end

	if (percentW == initial_p) then
		for i = 1, column_count do
			if (i <= #T.tab_identifiers and percentW > ((i - 1) / column_count) and percentW < (i / column_count)) then
				timer:SetTimeout(1, function() gkinterface.GKProcessCommand("tabnext "..(i + T.tab_window_offset)) end)
				break
			end
		end
	end
	initial_p = 0
end

T.Events.TOUCH_PRESSED = function(_, event, id, percentW, percentH, posX, posY)
	if     (id == T.Regions.list) then
		list_press(event, id, percentW, percentH)
		initial_p = percentH
	elseif (id == T.Regions.filter) then
		initial_p = percentW
	elseif (id == T.Regions.tab) then
		initial_p = percentW
	end
end

T.Events.TOUCH_RELEASED = function(_, event, id, percentW, percentH, posX, posY)
	if     (id == T.Regions.list) then
		list_release(event, id, percentW, percentH)
	elseif (id == T.Regions.filter) then
		filter_release(event, id, percentW, percentH)
	elseif (id == T.Regions.tab) then
		tab_release(event, id, percentW, percentH)
	end
end

RegisterEvent(T.Events, "TOUCH_PRESSED")
RegisterEvent(T.Events, "TOUCH_RELEASED")
